@@ -74,10 +74,10 @@ def parse_cmdlineoptions(arglist):
 		  "include-globbing-filelist-stdin", "include-regexp=",
 		  "include-special-files", "include-symbolic-links",
 		  "list-at-time=", "list-changed-since=", "list-increments",
-		  "list-increment-sizes", "never-drop-acls", "no-acls",
-		  "no-carbonfile", "no-compare-inode", "no-compression",
-		  "no-compression-regexp=", "no-eas", "no-file-statistics",
-		  "no-hard-links", "null-separator",
+		  "list-increment-sizes", "never-drop-acls",
+		  "no-acls", "no-carbonfile",
+		  "no-compare-inode", "no-compression", "no-compression-regexp=",
+		  "no-eas", "no-file-statistics", "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
 		  "preserve-numerical-ids", "print-statistics",
 		  "remote-cmd=", "remote-schema=",
@@ -87,7 +87,7 @@ def parse_cmdlineoptions(arglist):
 		  "user-mapping-file=", "verbosity=", "verify",
 		  "verify-at-time=", "version"])
 	except getopt.error, e:
-		commandline_error("Bad commandline options: %s" % str(e))
+		commandline_error("Bad commandline options: " + str(e))
 
 	for opt, arg in optlist:
 		if opt == "-b" or opt == "--backup-mode": action = "backup"
@@ -231,9 +231,8 @@ def final_set_action(rps):
 	else: action = "backup"
 
 def commandline_error(message):
-	sys.stderr.write("Error: %s\n" % message)
-	sys.stderr.write("See the rdiff-backup manual page for instructions\n")
-	sys.exit(2)
+	Log.FatalError(message + "\nSee the rdiff-backup manual page for "
+				   "more information.")
 
 def misc_setup(rps):
 	"""Set default change ownership flag, umask, relay regexps"""
@@ -291,6 +290,19 @@ def cleanup():
 	Log.close_logfile()
 	if not Globals.server: SetConnections.CloseConnections()
 
+def error_check_Main(arglist):
+	"""Run Main on arglist, suppressing stack trace for routine errors"""
+	try: Main(arglist)
+	except SystemExit: raise
+	except Exception, exc:
+		errmsg = robust.is_routine_fatal(exc)
+		if errmsg:
+			Log.exception(2, 6)
+			Log.FatalError(errmsg)
+		else:
+			Log.exception(2, 2)
+			raise
+
 def Main(arglist):
 	"""Start everything up!"""
 	parse_cmdlineoptions(arglist)
@@ -304,7 +316,6 @@ def Main(arglist):
 	cleanup()
 	if return_val is not None: sys.exit(return_val)
 
-
 def Backup(rpin, rpout):
 	"""Backup, possibly incrementally, src_path to dest_path."""
 	global incdir
@@ -647,17 +658,24 @@ def RemoveOlderThan(rootrp):
 	"""Remove all increment files older than a certain time"""
 	rootrp = require_root_set(rootrp, 0)
 	rot_require_rbdir_base(rootrp)
-	try: time = Time.genstrtotime(remove_older_than_string)
+
+	time = rot_check_time(remove_older_than_string)
+	if time is None: return
+	Log("Actual remove older than time: %s" % (time,), 6)
+	manage.delete_earlier_than(Globals.rbdir, time)
+
+def rot_check_time(time_string):
+	"""Check remove older than time_string, return time in seconds"""
+	try: time = Time.genstrtotime(time_string)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	timep = Time.timetopretty(time)
-	Log("Deleting increment(s) before %s" % timep, 4)
 
 	times_in_secs = [inc.getinctime() for inc in 
 		  restore.get_inclist(Globals.rbdir.append_path("increments"))]
 	times_in_secs = filter(lambda t: t < time, times_in_secs)
 	if not times_in_secs:
-		Log.FatalError("No increments older than %s found, exiting."
-					   % (timep,), 1, errlevel = 0)
+		Log("No increments older than %s found, exiting." %
+			(Time.timetopretty(time),), 3)
+		return None
 
 	times_in_secs.sort()
 	inc_pretty_time = "\n".join(map(Time.timetopretty, times_in_secs))
@@ -665,17 +683,16 @@ def RemoveOlderThan(rootrp):
 		Log.FatalError("Found %d relevant increments, dated:\n%s"
 			"\nIf you want to delete multiple increments in this way, "
 			"use the --force." % (len(times_in_secs), inc_pretty_time))
-
 	if len(times_in_secs) == 1:
 		Log("Deleting increment at time:\n" + inc_pretty_time, 3)
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
-	manage.delete_earlier_than(Globals.rbdir, time)
+	return times_in_secs[-1]+1 # make sure we don't delete current increment
 
 def rot_require_rbdir_base(rootrp):
 	"""Make sure pointing to base of rdiff-backup dir"""
 	if restore_index != ():
-		Log.FatalError("Increments for directory %s cannot be removed separately.\n"
-					   "Instead run on entire directory %s." %
+		Log.FatalError("Increments for directory %s cannot be removed "
+					   "separately.\nInstead run on entire directory %s." %
 					   (rootrp.path, restore_root.path))
 
 
