@@ -164,7 +164,9 @@ class FSAbilities:
 	def set_hardlinks(self, testdir):
 		"""Set self.hardlinks to true iff hard linked files can be made"""
 		hl_source = testdir.append("hardlinked_file1")
-		hl_dest = testdir.append("hardlinked_file2")
+		hl_dir = testdir.append("hl")
+		hl_dir.mkdir()
+		hl_dest = hl_dir.append("hardlinked_file2")
 		hl_source.touch()
 		try:
 			hl_dest.hardlink(hl_source.path)
@@ -197,7 +199,8 @@ class FSAbilities:
 		assert ord_rp.lstat()
 		ord_rp.delete()
 
-		extended_filename = ':\\' + chr(175)
+		# Try a UTF-8 encoded character
+		extended_filename = ':\\ ' + chr(225) + chr(132) + chr(137)
 		try:
 			ext_rp = subdir.append(extended_filename)
 			ext_rp.touch()
@@ -421,8 +424,8 @@ class SetGlobals:
 		self.update_triple(self.src_fsa.acls, self.dest_fsa.acls,
 						   ('acls_active', 'acls_write', 'acls_conn'))
 		if Globals.never_drop_acls and not Globals.acls_active:
-			Log.FatalError("--never-drop-acls specified, but ACL support\n"
-						   "missing from destination filesystem")
+			log.Log.FatalError("--never-drop-acls specified, but ACL support\n"
+							   "missing from destination filesystem")
 
 	def set_resource_forks(self):
 		self.update_triple(self.src_fsa.resource_forks,
@@ -433,9 +436,6 @@ class SetGlobals:
 	def set_carbonfile(self):
 		self.update_triple(self.src_fsa.carbonfile, self.dest_fsa.carbonfile,
 			  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
-		if self.src_fsa.carbonfile and not Globals.carbonfile_active:
-			Log("Source may have carbonfile support, but support defaults to "
-				"off.\n  Use --carbonfile to enable.", 5)
 
 	def set_hardlinks(self):
 		if Globals.preserve_hardlinks != 0:
@@ -510,9 +510,11 @@ class BackupSetGlobals(SetGlobals):
 			log.Log("Warning: File system no longer needs quoting, "
 					"but we will retain for backwards compatibility.", 2)
 			return actual_ctq
-		log.Log.FatalError("""New quoting requirements!
+		if Globals.chars_to_quote is None:
+			log.Log.FatalError("""New quoting requirements!
 
-The quoting this session appears to need do not match those in
+The quoting chars this session needs (%s) do not match
+the repository settings (%s) listed in
 
 %s
 
@@ -520,10 +522,7 @@ This may be caused when you copy an rdiff-backup repository from a
 normal file system onto a windows one that cannot support the same
 characters, or if you backup a case-sensitive file system onto a
 case-insensitive one that previously only had case-insensitive ones
-backed up onto it.
-
-If you want to risk it, remove the file
-rdiff-backup-data/chars_to_quote.""" % (ctq_rp.path,))
+backed up onto it.""" % (suggested_ctq, actual_ctq, ctq_rp.path))
 
 
 class RestoreSetGlobals(SetGlobals):
